package es.ua.jtech.cw.sesion09.ranking.filter;

import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import es.ua.jtech.cw.sesion09.ranking.dao.DAOException;
import es.ua.jtech.cw.sesion09.ranking.dao.PaginasDAO;
import es.ua.jtech.cw.sesion09.ranking.parser.AnalisisHTML;
import es.ua.jtech.cw.sesion09.ranking.wrapper.GenericResponseWrapper;

public final class AccesoPaginaFilter implements Filter {

	private FilterConfig filterConfig = null;

	/**
	 * Efectua el filtrado de una peticin.
	 * 
	 * @param request
	 *            Peticin realizada al servidor
	 * @param response
	 *            Respuesta para enviar al cliente
	 * @param chain
	 *            Cadena de filtros para procesar la peticin
	 * @throws  
	 */

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		HttpServletRequest http_request = null;
		HttpServletResponse http_response = null;

		if (request instanceof HttpServletRequest) {
			http_request = (HttpServletRequest) request;
		}

		if (response instanceof HttpServletResponse) {
			http_response = (HttpServletResponse) response;
		}

		// Tratamos solo peticiones HTTP

		if (http_request != null && http_response != null) {

			ServletContext sc = filterConfig.getServletContext();
			PaginasDAO pdao = new PaginasDAO();
			String url = http_request.getRequestURI();

			// Creamos un wrapper

			GenericResponseWrapper responseWrapper =
				new GenericResponseWrapper(http_response);

			// Efectuamos la peticion al recurso

			chain.doFilter(http_request, responseWrapper);
			byte[] datos = responseWrapper.getData();

			try {
				if(pdao.estaRegistrada(url)) {
					// Si la pagina solicitada esta registrada, incrementa el numero de accesos
					pdao.incrementaAccesos(url);
				} else {
					// Si no, registra la pagina (ruta, titulo) en la base de datos
					String responseString = new String(datos);
					AnalisisHTML parser = new AnalisisHTML();
					parser.parse(responseString.toCharArray());

					pdao.registraPagina(url, parser.getTitulo());					
				}
				
			} catch (DAOException e) {
				sc.log("Error al registrar acceso: " + e.getMessage());
			}

			// Establecemos cabeceras para evitar cache
			http_response.setHeader("Cache-control", "no-cache");
			http_response.setHeader("Cache-control", "no-store");
			http_response.setHeader("Pragma", "no-cache");
			http_response.setHeader("Expires", "0");
			
			// Devolvemos el contenido al cliente

			OutputStream out = http_response.getOutputStream();
			out.write(datos);
			out.close();

		} else {
			chain.doFilter(request, response);
		}
	}

	/**
	 * Destruye el filtro. Cierra conexin a base de datos
	 */

	public void destroy() {
		this.filterConfig = null;
	}

	/**
	 * Inicializa el filtro. Abre conexin a BD y lee parmetros.
	 * 
	 * @param filterConfig
	 *            Configuracin del filtro y de la aplicacin.
	 */

	public void init(FilterConfig filterConfig) throws ServletException {
		this.filterConfig = filterConfig;
	}

}
